Package: EpiDISH
Title: Epigenetic Dissection of Intra-Sample-Heterogeneity
Version: 2.28.0
Description: EpiDISH is a R package to infer the proportions of a
        priori known cell-types present in a sample representing a
        mixture of such cell-types. Right now, the package can be used on DNAm 
        data of blood-tissue of any age, from birth to old-age, generic epithelial
	tissue and breast tissue. Besides, the package provides a function that allows 
        the identification of differentially methylated cell-types and their 
        directionality of change in Epigenome-Wide Association Studies.
Date: 2026-03-13
Authors@R: c(person("Andrew E.", "Teschendorff", role=c("aut"), email="andrew@picb.ac.cn"),
        person("Shijie C.", "Zheng", role=c("aut", "cre"), email="shijieczheng@gmail.com"))
Depends: R (>= 4.1)
Imports: MASS, e1071, quadprog, parallel, stats, matrixStats, stringr,
        locfdr, Matrix, genefilter
Suggests: roxygen2, GEOquery, BiocStyle, knitr, rmarkdown, Biobase,
        testthat
VignetteBuilder: knitr
License: GPL-2
LazyData: true
RoxygenNote: 7.1.2
URL: https://github.com/sjczheng/EpiDISH
BugReports: https://github.com/sjczheng/EpiDISH/issues
biocViews: DNAMethylation, MethylationArray, Epigenetics,
        DifferentialMethylation, ImmunoOncology
git_url: https://git.bioconductor.org/packages/EpiDISH
git_branch: RELEASE_3_23
git_last_commit: 2574adb
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 22:05:18 UTC; biocbuild
Author: Andrew E. Teschendorff [aut],
  Shijie C. Zheng [aut, cre]
Maintainer: Shijie C. Zheng <shijieczheng@gmail.com>
Built: R 4.6.0; ; 2026-04-29 04:41:52 UTC; unix

# Generated by roxygen2: do not edit by hand

export(CellDMC)
export(ConstructDNAmPanel)
export(DoMetaEfron)
export(epidish)
export(hepidish)
import(locfdr)
import(matrixStats)
import(parallel)
import(stats)
importFrom(MASS,rlm)
importFrom(Matrix,rankMatrix)
importFrom(e1071,svm)
importFrom(genefilter,fastT)
importFrom(matrixStats,rowMaxs)
importFrom(matrixStats,rowMeans2)
importFrom(matrixStats,rowMedians)
importFrom(matrixStats,rowMins)
importFrom(quadprog,solve.QP)
importFrom(stats,p.adjust)
importFrom(stats,p.adjust.methods)
importFrom(stats,pt)
importFrom(stringr,str_c)
importFrom(utils,head)

#' @title A function that allows the identification of differentially methylated 
#' cell-types in in Epigenome-Wide Association Studies(EWAS)
#' 
#' @aliases CellDMC
#' 
#' @description 
#' An outstanding challenge of Epigenome-Wide Association Studies performed in 
#' complex tissues is the identification of the specific cell-type(s) 
#' responsible for the observed differential methylation. CellDMC is a novel 
#' statistical algorithm, which is able to identify not only differentially 
#' methylated positions, but also the specific cell-type(s) driving the 
#' methylation change. 
#' 
#' @param beta.m
#' A beta value matrix with rows labeling the CpGs and columns labeling 
#' samples. 
#' 
#' @param pheno.v
#' A vector of phenotype. CellDMC can handle both of binary and 
#' continuous/oderinal phenotypes. \code{NA} is not allowed in
#' \code{pheno.v}.
#' 
#' @param frac.m
#' A matrix contains fractions of each cell-type. Each row labels a sample, 
#' with the same order of the columns in beta.m. Each column labels a 
#' cell-type. Column names, which are the names of cell-types, are required. The 
#' rowSums of frac.m should be 1 or close to 1.
#' 
#' @param adjPMethod
#' The method used to adjust p values. The method can be any of method
#'  accepted by \code{\link{p.adjust}}.
#' 
#' @param adjPThresh
#' A numeric value, default as 0.05. This is used to call DMCTs. 
#' For each cell-type respectively, the CpG with the adjusted p values less than
#'  this threshold will be reported as DMCTs (-1 or 1) in the 'dmct' matrix in 
#'  the returned list.
#' 
#' @param cov.mod
#' A design matrix from \code{model.matrix}, which contains other covariates to
#' be adjusted. For example, input 
#' \code{model.matrix(~ geneder, data = pheno.df)} to adjust gender. Do not put
#'  cell-type fraction here!
#' 
#' @param sort
#' Default as \code{FALSE}. If \code{TRUE}, the data.frame in coe list will be
#' sorted based on p value of each CpG. The order of rows in 'dmct' will not 
#' change since the orders of each cell-type are different.
#' 
#' @param mc.cores
#' The number of cores to use, i.e. at most how many threads will 
#' run simultaneously. The defatul is 1, which means no parallelization. 
#' 
#' @return A list with the following two items. 
#' 
#' @return dmct
#' A matrix gives wheter the input CpGs are DMCTs and DMCs. The first column 
#' tells whether a CpG is a DMC or not. If the CpG is called as DMC, the value 
#' will be 1, otherwise it is 0. The following columns give DMCTs for each 
#' cell-type. If a CpG is a DMCT, the value will be 1 (hypermethylated for case 
#' compared to control) or -1 (hypomethylated for case compared to control). 
#' Otherwise, the value is 0 (non-DMCT). The rows of this matrix are ordered as
#' the same as that of the input \code{beta.m}. 
#' 
#' @return coe
#' This list contains several dataframes, which correspond to each cell-type in
#'  \code{frac.m}. Each dataframe contains all CpGs in input \code{beta.m}. 
#'  All dataframes contain estimated DNAm changes (\code{Estimate}), 
#'  standard error (\code{SE}), estimated t statistics (\code{t}), 
#'  raw P values (\code{p}), and multiple hypothesis corrected P 
#'  values (\code{adjP}).
#' 
#' 
#' @references 
#' Zheng SC, Breeze CE, Beck S, Teschendorff AE. 
#' \emph{Identification of differentially methylated cell-types in 
#' Epigenome-Wide Association Studies.} 
#' Nat Methods (2018) 15: 1059-1066
#' doi:\href{https://doi.org/10.1038/s41592-018-0213-x}{10.1038/s41592-018-0213-x}.
#' 
#' @examples 
#' data(centEpiFibIC.m)
#' data(DummyBeta.m)
#' out.l <- epidish(DummyBeta.m, centEpiFibIC.m, method = 'RPC')
#' frac.m <- out.l$estF
#' pheno.v <- rep(c(0, 1), each = 5)
#' celldmc.o <- CellDMC(DummyBeta.m, pheno.v, frac.m) 
#' # Pls note this is a faked beta value matrix.
#' 
#' 
#' @import matrixStats
#' @import parallel
#' @importFrom stringr str_c
#' @importFrom Matrix rankMatrix
#' @import stats
#' 
#' @export
#' 
CellDMC <- function(beta.m, pheno.v, frac.m, 
                    adjPMethod = "fdr", adjPThresh = 0.05, cov.mod = NULL, 
                    sort = FALSE, mc.cores = 1) {
    ### check input
    if (sum(is.na(pheno.v)) > 0) stop("No NA allowed in pheno.v!")
    if (ncol(beta.m) != length(pheno.v)) 
        stop("Number of columns of beta.m should equal to length of pheno.v!")
    if (ncol(beta.m) != nrow(frac.m)) 
        stop("Number of columns of beta.m should equal to number of rows of frac.m!")
    if (length(colnames(frac.m)) != ncol(frac.m)) 
        stop("Pls assign correct names of cell-type to frac.m")
  
    ### check whether input is beta value matrix
    is.beta <- ((min(beta.m) >= 0) & (max(beta.m) <= 1))
    
    ### guess factor input
    if (nlevels(factor(pheno.v)) == 2) {
        message("Binary phenotype detected. Predicted change will be 1 - 0.")
        pheno.v <- factor(pheno.v)
    }
    if (!is.factor(pheno.v) & !is.character(pheno.v)) 
        message("pheno.v is not factor or character. Treating as continuous variables.")

  ### Fit model
  design <- model.matrix(~ frac.m + pheno.v:frac.m)[, -1]
  if (Matrix::rankMatrix(design) < ncol(design)) {
    stop("The design matrix is not full ranked.\nThis means that you coundn't make inference for all cell-types in your fraction matrix.
         This is usally casued by fractions of a cell-type of one pheno type are all 0 or some fractions in one pheno type are paralle to that of another cell-type.
         You might use which(colSums(model.matrix(~ frac.m + pheno.v:frac.m)[, -1]) == 0) to find the cell type.")
  }
  
  
  if (!is.null(cov.mod)) design <- cbind(design, cov.mod[, -1])
  IntNames.v <- str_c(colnames(frac.m), "Pheno")
  colnames(design)[(1 + ncol(frac.m)):(2*ncol(frac.m))] <- IntNames.v 
  
  ### fit linear model for each CpG
  allCoe.m <- do.call(rbind, mclapply(seq_len(nrow(beta.m)), function(i) {
      beta.v <- beta.m[i, ]
      ### model
      Int.o <- lm(beta.v ~ .-1, data = data.frame(design))
    
      ### get coe
      IntCoe.m <- summary(Int.o)$coe[IntNames.v, ]
      IntCoe.v <- unlist(apply(IntCoe.m, 1, function(x) list(x)))
    
      names(IntCoe.v) <- NULL
      return(IntCoe.v)
  }, mc.preschedule = TRUE, mc.cores = mc.cores, mc.allow.recursive = TRUE))
  
  ### extract coefficients for each cell-type
  coe.ld <- lapply(seq_len(ncol(frac.m)), function(j) {
        idx <- ((j - 1)*4 + 1):((j - 1)*4 + 4)
        tmp.m <- allCoe.m[, idx]
        tmp.m <- cbind(tmp.m, p.adjust(tmp.m[, 4], method = adjPMethod))
        if (is.beta) { 
            tmp.m[which(tmp.m[,1] > 1),1] <- 1
            tmp.m[which(tmp.m[,1] < -1),1] <- -1
        }  ### if input is a beta values matrix, bound the estimated changes

        colnames(tmp.m) <- c("Estimate", "SE", "t", "p", "adjP")
        rownames(tmp.m) <- rownames(beta.m)
        return(data.frame(tmp.m))
  })
  names(coe.ld) <- colnames(frac.m)
  
  ### get dmct matrix
  dmct.m <- matrix(rep(0, ncol(frac.m)*nrow(beta.m)), ncol = ncol(frac.m))
  dmct.idx <- which(sapply(coe.ld, "[[", "adjP") < adjPThresh)
  dmct.m[dmct.idx] <- sign(sapply(coe.ld, "[[", "Estimate")[dmct.idx])
  dmc.v <- ifelse(rowAlls(dmct.m == 0), 0, 1)
  dmct.m <- cbind(dmc.v, dmct.m)
  colnames(dmct.m) <- c("DMC", colnames(frac.m))
  rownames(dmct.m) <- rownames(beta.m)
  
  if(sort) coe.ld <- lapply(coe.ld, function(x) x[order(x$p),] )
  
  return(list(dmct = dmct.m, coe = coe.ld))
}

#' @title An R-function to perform a meta-analysis over multiple studies using 
#' an empirical Bayes procedure by Efron followed by Stouffer method.
#' 
#' @aliases DoMetaEfron
#' 
#' @description
#' An R-function to perform a meta-analysis over multiple studies using 
#' an empirical Bayes procedure by Efron followed by Stouffer method.
#' 
#' @param stat.m
#' A matrix of signed statistics (e.g. t-statistics) with rows labeling genomic 
#' features (e.g. CpGs or genes) and columns labeling studies. rownames must be 
#' provided. 
#' 
#' @param pval.m
#' A matrix matched to stat.m containing the associated P-values, with rows 
#' labeling genomic features (e.g. CpGs or genes) and columns labeling studies.
#' 
#' @param bre
#' The number of breakpoints to divide statistics per study into bins. By 
#' default this is 120. See input argument for locfdr function from locfdr 
#' package.
#' 
#' @param df 
#' The number of degrees of freedom for fitting spline. By default this is 
#' 15. See input argument for locfdr function from locfdr package.
#' 
#' @param pct0
#' Percentage of statistics to use for fitting null. By default this is 0.25 
#' (i.e. 25\%).
#' 
#' @param plotlocfdr
#' Determines whether to plot output or not. By default this is set to 0 meaning
#'  no plot. See input argument for locfdr function from locfdr package.
#' 
#' @return meta.m
#' A matrix with rows as in stat.m, and with 3 columns labeling Stouffer's 
#' z-statistic, P-value and Benjamini-Hochberg adjusted P-value.
#' 
#' @import locfdr
#' 
#' @export
#' 

DoMetaEfron <- function(stat.m, pval.m, bre = 120, df = 15, pct0 = 0.25, plotlocfdr = 0){
  
  ### first transform p-values and signed t-stats to z-scores
  zMeta.m <- stat.m;
  zMeta.m[stat.m>0] <- qnorm(pval.m[stat.m>0]/2,lower.tail=FALSE);
  zMeta.m[stat.m<0] <- qnorm(pval.m[stat.m<0]/2,lower.tail=TRUE);
  
  ### for each study estimate z-stats of null
  zmodMeta.m <- zMeta.m;
  for(st in 1:ncol(zMeta.m)){
    locfdr.o <- locfdr(zMeta.m[,st], bre=bre, df=df, pct=0, pct0=pct0, nulltype=1, type=0, plot = plotlocfdr, sw=0);
    mu <- locfdr.o$fp0[3,1];
    sd <- locfdr.o$fp0[3,2];
    zmodMeta.m[,st] <- (zMeta.m[,st] - mu)/sd;
  }
  
  ### stouffer meta
  zMeta.v <- apply(zmodMeta.m,1,sum)/sqrt(ncol(stat.m));
  pMeta.v <- 2*(1-pnorm(abs(zMeta.v)));
  padjMeta.v <- p.adjust(pMeta.v,"BH");
  
  meta.m <- cbind(zMeta.v,pMeta.v,padjMeta.v);
  rownames(meta.m) <- rownames(stat.m);
  colnames(meta.m) <- c("z(Stouffer)","P(Stouffer)","AdjP(BH)");
  
  return(meta.m);
} ### EOF

#' @title 
#' Hierarchical EpiDISH (HEpiDISH)
#' 
#' @aliases hepidish
#'  
#' @description 
#' HEpiDISH is an iterative hierarchical procedure of EpiDISH. HEpiDISH uses two 
#' distinct DNAm references, a primary reference for the estimation of several 
#' cell-types fractions, and a separate secondary non-overlapping DNAm reference 
#' for the estimation of underlying subtype fractions of one of the cell-type in 
#' the primary reference.
#' 
#' 
#' @param beta.m
#' A data matrix with rows labeling the molecular features (should use same ID 
#' as in reference matrices) and columns labeling samples (e.g. primary tumour specimens). 
#' Missing value is not allowed and all values should be positive or zero. 
#' In the case of DNA methylation, these are beta-values.
#' 
#' @param ref1.m
#' A matrix of \strong{primary} reference 'centroids', i.e. representative molecular profiles, 
#' for a number of cell subtypes. rows label molecular features (e.g. CpGs,...) 
#' and columns label the cell-type. IDs need to be provided as rownames and 
#' colnames, respectively. Missing value is not allowed, and all values in 
#' this matrix should be positive or zero. For DNAm data, values should be 
#' beta-values.
#' 
#' @param ref2.m
#' Similar to \code{ref1.m}, but now a A matrix of \strong{secondary} reference.
#' For example, \code{ref1.m} contains reference centroids for epithelial cells,
#' fibroblasts and total immune cells. \code{ref2.m} can be subtypes of immune
#' cells, such as B-cells, NK cells, monocytes and etc. 
#' 
#' @param h.CT.idx
#' A index tells which cell-type in \code{ref1.m} is the higher order cell-types 
#' in \code{ref2.m}. For example, \code{ref1.m} contains reference centroids for 
#' epithelial cells, fibroblasts and total immune cells. \code{ref2.m} contains 
#' subtypes of immune cells, the \code{h.CT.idx} should be 3, corresponding to
#' immune cells in \code{ref1.m}.
#' 
#' 
#' @param method
#' Chioce of a reference-based method ('RPC','CBS','CP')
#' 
#' @param maxit
#' Only used in RPC mode, the limit of the number of IWLS iterations
#' 
#' @param nu.v
#' Only used in CBS mode. It is a vector of several candidate nu values. nu is 
#' parameter needed for nu-classification, nu-regression, and 
#' one-classification in svm. The best estimation results among all candidate nu 
#' will be automatically returned.
#' 
#' @param constraint
#' Only used in CP mode, you can choose either of 'inequality' or 'equality' 
#' normalization constraint. The default is 'inequality' (i.e sum of weights 
#' adds to a number less or equal than 1), which was implemented in 
#' Houseman et al (2012).
#' 
#' @return A matrix of the estimated fractions
#' 
#' 
#' @references 
#' Zheng SC, Webster AP, Dong D, Feber A, Graham DG, Sullivan R, Jevons S, Lovat LB, 
#' Beck S, Widschwendter M, Teschendorff AE
#' \emph{A novel cell-type deconvolution algorithm reveals substantial contamination by immune cells in saliva, buccal and cervix.}
#' Epigenomics (2018) 10: 925-940.
#' doi:\href{https://doi.org/10.2217/epi-2018-0037}{
#' 10.2217/epi-2018-0037}.
#' 
#' Teschendorff AE, Breeze CE, Zheng SC, Beck S. 
#' \emph{A comparison of reference-based algorithms for correcting cell-type 
#' heterogeneity in Epigenome-Wide Association Studies.}
#' BMC Bioinformatics (2017) 18: 105.
#' doi:\href{https://doi.org/10.1186/s12859-017-1511-5}{
#' 10.1186/s12859-017-1511-5}.
#' 
#' Houseman EA, Accomando WP, Koestler DC, Christensen BC, Marsit CJ, 
#' Nelson HH, Wiencke JK, Kelsey KT. 
#' \emph{DNA methylation arrays as surrogate measures of cell mixture 
#' distribution.} 
#' BMC Bioinformatics (2012) 13: 86.
#' doi:\href{https://doi.org/10.1186/1471-2105-13-86}{10.1186/1471-2105-13-86}.
#' 
#' Newman AM, Liu CL, Green MR, Gentles AJ, Feng W, Xu Y, Hoang CD, Diehn M, 
#' Alizadeh AA. 
#' \emph{Robust enumeration of cell subsets from tissue expression profiles.}
#' Nat Methods (2015) 12: 453-457.
#' doi:\href{https://doi.org/10.1038/nmeth.3337}{10.1038/nmeth.3337}.
#' 
#' @examples 
#' data(centEpiFibIC.m)
#' data(centBloodSub.m)
#' data(DummyBeta.m)
#' frac.m <- hepidish(beta.m = DummyBeta.m, ref1.m = centEpiFibIC.m, 
#' ref2.m = centBloodSub.m, h.CT.idx = 3, method = 'RPC')
#' 
#' 
#' @export
#'     

hepidish <- function(beta.m, ref1.m, ref2.m, h.CT.idx, method = c("RPC", "CBS", "CP"), 
    maxit = 50, nu.v = c(0.25, 0.5, 0.75), constraint = c("inequality", "equality")) {
    method <- match.arg(method)
    constraint <- match.arg(constraint)
    if (!method %in% c("RPC", "CBS", "CP")) 
        stop("Input a valid method!")
    if (method == "RPC") {
        frac1.m <- DoRPC(beta.m, ref1.m, maxit)$estF
        frac2.m <- DoRPC(beta.m, ref2.m, maxit)$estF
        frac.m <- cbind(frac1.m[, -h.CT.idx, drop = FALSE], frac1.m[, h.CT.idx] * frac2.m)
    } else if (method == "CBS") {
        frac1.m <- DoCBS(beta.m, ref1.m, nu.v)$estF
        frac2.m <- DoCBS(beta.m, ref2.m, nu.v)$estF
        frac.m <- cbind(frac1.m[, -h.CT.idx, drop = FALSE], frac1.m[, h.CT.idx] * frac2.m)
    } else if (method == "CP") {
        if (!constraint %in% c("inequality", "equality")) {
            # make sure constraint must be inequality or equality
            stop("constraint must be inequality or equality when using CP!")
        } else {
            frac1.m <- DoCP(beta.m, ref1.m, constraint)$estF
            frac2.m <- DoCP(beta.m, ref2.m, constraint)$estF
            frac.m <- cbind(frac1.m[, -h.CT.idx, drop = FALSE], frac1.m[, h.CT.idx] * frac2.m)
        }
    }
    return(frac.m)
}

#' @title Construct DNA Methylation reference Panel
#'
#' @description Constructs a cell-type-specific DNA methylation panel by selecting
#'              top-ranked hypo-methylated probes based on a "Gap Specificity Score" metric,
#'              which prioritizes markers with large effect sizes.
#'
#' @param dnam.matrix A DNA methylation beta value matrix with no NA values.
#'                    Rows should be Probe IDs and columns should be samples.
#' @param cell.types.info A data frame that must contain a 'CellType' column,
#'                        specifying the cell type for each sample in `dnam.matrix`.
#' @param panel.cell.types A character vector specifying for which cell types a
#'                         panel should be constructed. These must be present in
#'                         `cell.types.info$CellType`.
#' @param n.probes The number of top-ranked hypo-methylated probes to select for
#'                 each cell type. Default is 100.
#' @param fdr.threshold The FDR (False Discovery Rate) threshold for filtering
#'                      significantly differentially methylated probes. Default is 0.05.
#' @param p.adjust.method The method for multiple testing correction. Must be one of
#'                        the methods supported by `stats::p.adjust`. Default is "BH".
#' @param summary.method The method to use for summarizing methylation values when
#'                       creating the reference panel. Can be "mean" or "median".
#'                       Default is "mean".
#' @param equal.variance A logical value (`TRUE` or `FALSE`) passed to `genefilter::fastT`, 
#'                       indicating whether to assume equal variance between groups. Default is FALSE.
#'
#' @return A list containing two components:
#'         \item{DetailedResults}{A list where each element is a data frame for a
#'                                cell type in `panel.cell.types`. The data frame
#'                                contains detailed information for the selected probes,
#'                                such as GapScore, AbsEffectSize, Zstatistic, and Pvalue.}
#'         \item{ReferencePanel}{A matrix with probes as rows and cell types as columns.
#'                               The values are the mean or median methylation beta values
#'                               for each probe in the corresponding cell type. This can serve
#'                               as a reference matrix for deconvolution.}
#'
#' @details
#' The function's logic is as follows:
#' 1. For each target cell type, it is compared against all other cell types.
#' 2. A t-test is used to find differentially methylated probes, and p-values are
#'    adjusted to FDR using the specified `p.adjust.method`.
#' 3. Probes are filtered based on the `fdr.threshold`.
#' 4. Only probes that are hypo-methylated in the target cell type (negative t-statistic)
#'    are retained.
#' 5. A "Gap specificity Score" is calculated for these probes (defined as the minimum beta value
#'    in other cell types minus the maximum beta value in the target cell type).
#' 6. Probes are ranked by the Gap Score, and the top `n.probes` are selected.
#' 7. Finally, the probes selected for all target cell types are combined to form the
#'    final panel, and the reference matrix is computed using the specified `summary.method`.
#'
#' @importFrom genefilter fastT
#' @importFrom stats p.adjust pt p.adjust.methods
#' @importFrom matrixStats rowMaxs rowMins rowMeans2 rowMedians
#' @importFrom utils head
#' @export
#'
#' @examples
#' \dontrun{
#' # Assuming you have referenceBetas_matrix_noNA and referenceCovars_df
#' # library(genefilter)
#' # library(stats)
#' # library(matrixStats)
#' 
#' # Define target cell types for the panel
#' target_cell_types <- c("CD4T", "Bcell", "Mono")
#' 
#' # Construct the Panel
#' panel_results <- ConstructDNAmPanel(
#'   dnam.matrix = referenceBetas_matrix_noNA,
#'   cell.types.info = referenceCovars_df,
#'   panel.cell.types = target_cell_types,
#'   n.probes = 50,
#'   fdr.threshold = 0.05,
#'   p.adjust.method = "BH",
#'   summary.method = "median" 
#' )
#' 
#' # View the results
#' # View the reference panel matrix
#' head(panel_results$ReferencePanel)
#' 
#' # View detailed probe info for B-cells
#' head(panel_results$DetailedResults$Bcell)
#' }
ConstructDNAmPanel <- function(dnam.matrix, 
                               cell.types.info, 
                               panel.cell.types, 
                               n.probes = 100, 
                               fdr.threshold = 0.05, 
                               p.adjust.method = "BH",
                               summary.method = "mean",
                               equal.variance = FALSE) {
  
  # 1. Check for required packages
  if (!requireNamespace("genefilter", quietly = TRUE)) {
    stop("Package 'genefilter' is required but not installed.", call. = FALSE)
  }
  if (!requireNamespace("stats", quietly = TRUE)) {
    stop("Package 'stats' is required but not installed.", call. = FALSE)
  }
  if (!requireNamespace("matrixStats", quietly = TRUE)) {
    warning("Package 'matrixStats' is recommended for performance.", call. = FALSE)
    rowMeans2 <- function(x, ...) apply(x, 1, mean, ...)
    rowMaxs <- function(x, ...) apply(x, 1, max, ...)
    rowMins <- function(x, ...) apply(x, 1, min, ...)
  } else {
    rowMeans2 <- matrixStats::rowMeans2
    rowMaxs <- matrixStats::rowMaxs
    rowMins <- matrixStats::rowMins
    rowMedians <- matrixStats::rowMedians
  }
  
  # 2. Validate inputs
  if (!is.matrix(dnam.matrix)) dnam.matrix <- as.matrix(dnam.matrix)
  if (anyNA(dnam.matrix)) stop("'dnam.matrix' must be NA-free.")
  if (!"CellType" %in% colnames(cell.types.info)) stop("'cell.types.info' must have a 'CellType' column.")
  if (nrow(cell.types.info) != ncol(dnam.matrix)) stop("Row count of 'cell.types.info' must match column count of 'dnam.matrix'.")
  if (is.null(rownames(dnam.matrix))) stop("'dnam.matrix' must have probe IDs as rownames.")
  if (!all(panel.cell.types %in% cell.types.info$CellType)) {
    stop("Some 'panel.cell.types' not found in 'cell.types.info$CellType'.")
  }
  if (!summary.method %in% c("mean", "median")) {
    stop("'summary.method' must be either 'mean' or 'median'.")
  }
  if (summary.method == "median" && !requireNamespace("matrixStats", quietly = TRUE)) {
    stop("Package 'matrixStats' is required to use summary.method = 'median'.")
  }
  # --- New Validation Rules ---
  if (!is.logical(equal.variance) || length(equal.variance) != 1) {
    stop("'equal.variance' must be a single logical value (TRUE or FALSE).")
  }
  if (!p.adjust.method %in% stats::p.adjust.methods) {
    stop(paste("'p.adjust.method' is not valid. Choose one of:", 
               paste(stats::p.adjust.methods, collapse = ", ")))
  }
  # --------------------------
  
  message("Input validation complete.")
  
  # 3. Preprocess data
  tIndexes_all <- split(seq_len(ncol(dnam.matrix)), cell.types.info$CellType)
  
  cell_types_present_in_data <- intersect(panel.cell.types, names(tIndexes_all))
  if (length(cell_types_present_in_data) < 2) {
    stop("Fewer than two cell types are available to build the panel.")
  }
  
  message("Data preprocessing complete.")
  
  # 4. Core logic: select probes for each cell type
  detailed_results_list <- vector("list", length(cell_types_present_in_data))
  names(detailed_results_list) <- cell_types_present_in_data
  all_selected_probes <- character(0)
  
  for (target_ct_name in cell_types_present_in_data) {
    message("Processing cell type: ", target_ct_name)
    
    x1 <- tIndexes_all[[target_ct_name]]
    other_cts_names <- setdiff(names(tIndexes_all), target_ct_name)
    x2_all_others <- unlist(tIndexes_all[other_cts_names])
    
    if (length(x1) < 1 || length(x2_all_others) < 1) {
      warning(paste("Skipping", target_ct_name, "- zero samples in target or other groups."))
      next
    }
    
    df <- length(x1) + length(x2_all_others) - 2
    if (df <= 0) {
      warning(paste("Skipping", target_ct_name, "- non-positive degrees of freedom."))
      next
    }
    
    tstat_result <- tryCatch(genefilter::fastT(dnam.matrix, x1, x2_all_others, var.equal = equal.variance),
                             error = function(e) { warning(paste("t-test failed for", target_ct_name, ":", e$message)); NULL })
    
    if (is.null(tstat_result)) next
    
    p_values <- 2 * stats::pt(abs(tstat_result$z), df = df, lower.tail = FALSE)
    fdr <- stats::p.adjust(p_values, method = p.adjust.method)
    
    sig_probes_idx <- which(fdr < fdr.threshold)
    if (length(sig_probes_idx) == 0) {
      message("No significant probes for ", target_ct_name, " at FDR < ", fdr.threshold)
      next
    }
    
    p_sig <- dnam.matrix[sig_probes_idx, , drop = FALSE]
    tstat_sig_z <- tstat_result$z[sig_probes_idx]
    
    is_hypo <- sign(tstat_sig_z) < 0
    hypo_probes_idx_in_sig <- which(is_hypo)
    
    if (length(hypo_probes_idx_in_sig) == 0) {
      message("No hypo-methylated probes found for ", target_ct_name)
      next
    }
    
    max_target_vals <- rowMaxs(p_sig[hypo_probes_idx_in_sig, x1, drop = FALSE])
    min_other_samples <- rowMins(p_sig[hypo_probes_idx_in_sig, x2_all_others, drop = FALSE])
    gap_score_hypo <- min_other_samples - max_target_vals
    
    hypo_probes_in_sig <- rownames(p_sig)[hypo_probes_idx_in_sig]
    sorted_hypo_probes <- hypo_probes_in_sig[order(gap_score_hypo, decreasing = TRUE)]
    
    num_to_select <- min(n.probes, length(sorted_hypo_probes))
    final_selected_probes <- head(sorted_hypo_probes, num_to_select)
    
    if (length(final_selected_probes) == 0) next
    
    all_selected_probes <- c(all_selected_probes, final_selected_probes)
    
    selected_indices_in_sig <- match(final_selected_probes, rownames(p_sig))
    mean_target <- rowMeans2(p_sig[selected_indices_in_sig, x1, drop = FALSE])
    mean_all_others <- rowMeans2(p_sig[selected_indices_in_sig, x2_all_others, drop = FALSE])
    
    df_ct <- data.frame(
      ProbeID = final_selected_probes,
      GapScore = gap_score_hypo[match(final_selected_probes, hypo_probes_in_sig)],
      AbsEffectSize = abs(mean_target - mean_all_others),
      Zstatistic = tstat_sig_z[selected_indices_in_sig],
      Pvalue = p_values[sig_probes_idx][selected_indices_in_sig],
      row.names = final_selected_probes
    )
    detailed_results_list[[target_ct_name]] <- df_ct
  }
  
  # 5. Finalize results
  detailed_results_list <- detailed_results_list[!sapply(detailed_results_list, is.null)]
  candidatePanelProbes <- unique(all_selected_probes)
  
  if (length(candidatePanelProbes) == 0) {
    warning("No probes were selected for the final panel.")
    return(list(DetailedResults = detailed_results_list, ReferencePanel = matrix(0,0,0)))
  }
  
  # 6. Create the reference panel matrix
  summary.func <- if (summary.method == "mean") rowMeans2 else rowMedians
  
  p_candidate <- dnam.matrix[candidatePanelProbes, , drop = FALSE]
  processed_cell_types <- names(detailed_results_list)
  
  ReferencePanel <- matrix(NA, nrow = length(candidatePanelProbes), ncol = length(processed_cell_types))
  rownames(ReferencePanel) <- candidatePanelProbes
  colnames(ReferencePanel) <- processed_cell_types
  
  for (ct_name in processed_cell_types) {
    ind <- tIndexes_all[[ct_name]]
    if (length(ind) > 0) {
      ReferencePanel[, ct_name] <- summary.func(p_candidate[, ind, drop = FALSE])
    }
  }
  
  message("Panel construction complete. Returning results.")
  
  return(list(DetailedResults = detailed_results_list, 
              ReferencePanel = ReferencePanel))
}

#' @title 
#' Epigenetic Dissection of Intra-Sample-Heterogeneity
#' 
#' @aliases epidish
#'  
#' @description 
#' A reference-based function to infer the fractions of a priori known cell 
#' subtypes present in a sample representing a mixture of such cell-types. 
#' Inference proceeds via one of 3 methods (Robust Partial Correlations-RPC, 
#' Cibersort-CBS, Constrained Projection-CP), as determined by the user.
#' 
#' 
#' @param beta.m
#' A data matrix with rows labeling the molecular features (should use same ID 
#' as in ref.m) and columns labeling samples (e.g. primary tumour specimens). 
#' Missing value is not allowed and all values should be positive or zero. 
#' In the case of DNA methylation, these are beta-values.
#' 
#' @param ref.m
#' A matrix of reference 'centroids', i.e. representative molecular profiles, 
#' for a number of cell subtypes. rows label molecular features (e.g. CpGs,...) 
#' and columns label the cell-type. IDs need to be provided as rownames and 
#' colnames, respectively. Missing value is not allowed, and all values in 
#' this matrix should be positive or zero. For DNAm data, values should be 
#' beta-values.
#' 
#' @param method
#' Chioce of a reference-based method ('RPC','CBS','CP')
#' 
#' @param maxit
#' Only used in RPC mode, the limit of the number of IWLS iterations
#' 
#' @param nu.v
#' Only used in CBS mode. It is a vector of several candidate nu values. nu is 
#' parameter needed for nu-classification, nu-regression, and 
#' one-classification in svm. The best estimation results among all candidate nu 
#' will be automatically returned.
#' 
#' @param constraint
#' Only used in CP mode, you can choose either of 'inequality' or 'equality' 
#' normalization constraint. The default is 'inequality' (i.e sum of weights 
#' adds to a number less or equal than 1), which was implemented in 
#' Houseman et al (2012).
#' 
#' @return CP-mode
#' A list with the following entries: estF: a matrix of the estimated fractions; 
#' ref: the reference centroid matrix used; dataREF: the subset of the input 
#' data matrix with only the probes defined in the reference matrix.
#' 
#' @return CBS-mode
#' A list with the following entries: estF: a matrix of the estimated fractions; 
#' nu: a vector of 'best' nu-parameter for each sample; 
#' ref: the reference centroid matrix used;
#' dataREF: the subset of the input data matrix with only the probes defined in the 
#' reference matrix.
#' 
#' @return RPC-mode
#' A list with the following entries: estF: a matrix of the estimated fractions;
#'  ref: the reference centroid matrix used; 
#' dataREF: the subset of the input data matrix with only the probes defined in the 
#' reference matrix.
#' 
#' @references 
#' Teschendorff AE, Breeze CE, Zheng SC, Beck S. 
#' \emph{A comparison of reference-based algorithms for correcting cell-type 
#' heterogeneity in Epigenome-Wide Association Studies.}
#' BMC Bioinformatics (2017) 18: 105.
#' doi:\href{https://doi.org/10.1186/s12859-017-1511-5}{
#' 10.1186/s12859-017-1511-5}.
#' 
#' Houseman EA, Accomando WP, Koestler DC, Christensen BC, Marsit CJ, 
#' Nelson HH, Wiencke JK, Kelsey KT. 
#' \emph{DNA methylation arrays as surrogate measures of cell mixture 
#' distribution.} 
#' BMC Bioinformatics (2012) 13: 86.
#' doi:\href{https://doi.org/10.1186/1471-2105-13-86}{10.1186/1471-2105-13-86}.
#' 
#' Newman AM, Liu CL, Green MR, Gentles AJ, Feng W, Xu Y, Hoang CD, Diehn M, 
#' Alizadeh AA. 
#' \emph{Robust enumeration of cell subsets from tissue expression profiles.}
#' Nat Methods (2015) 12: 453-457.
#' doi:\href{https://doi.org/10.1038/nmeth.3337}{10.1038/nmeth.3337}.
#' 
#' @examples 
#' data(centDHSbloodDMC.m)
#' data(DummyBeta.m)
#' out.l <- epidish(DummyBeta.m, centDHSbloodDMC.m[,1:6], method = 'RPC')
#' frac.m <- out.l$estF
#' 
#' 
#' @export
#'     
epidish <- function(beta.m, ref.m, method = c("RPC", "CBS", "CP"), maxit = 50, nu.v = c(0.25, 
    0.5, 0.75), constraint = c("inequality", "equality")) {
    method <- match.arg(method)
    constraint <- match.arg(constraint)
    if (!method %in% c("RPC", "CBS", "CP")) 
        stop("Input a valid method!")
    if (method == "RPC") {
        out.o <- DoRPC(beta.m, ref.m, maxit)
    } else if (method == "CBS") {
        out.o <- DoCBS(beta.m, ref.m, nu.v)
    } else if (method == "CP") {
        if (!constraint %in% c("inequality", "equality")) {
            # make sure constraint must be inequality or equality
            stop("constraint must be inequality or equality when using CP!")
        } else out.o <- DoCP(beta.m, ref.m, constraint)
    }
    return(out.o)
}

### Reference-based methods

#' @importFrom MASS rlm
### RPC
DoRPC <- function(beta.m, ref.m, maxit) {
    map.idx <- match(rownames(ref.m), rownames(beta.m))
    rep.idx <- which(is.na(map.idx) == FALSE)
    data2.m <- beta.m[map.idx[rep.idx], , drop=FALSE]
    ref2.m <- ref.m[rep.idx, ]
    est.m <- matrix(nrow = ncol(data2.m), ncol = ncol(ref2.m))
    colnames(est.m) <- colnames(ref2.m)
    rownames(est.m) <- colnames(data2.m)
    for (s in seq_len(ncol(data2.m))) {
        rlm.o <- rlm(data2.m[, s] ~ ref2.m, maxit = maxit)
        coef.v <- summary(rlm.o)$coef[2:(ncol(ref2.m) + 1), 1]
        coef.v[which(coef.v < 0)] <- 0
        total <- sum(coef.v)
        coef.v <- coef.v/total
        est.m[s, ] <- coef.v
    }
    return(list(estF = est.m, ref = ref2.m, dataREF = data2.m))
}

#' @importFrom e1071 svm
### CIBERSORT
DoCBS <- function(beta.m, ref.m, nu.v) {
    map.idx <- match(rownames(ref.m), rownames(beta.m))
    rep.idx <- which(is.na(map.idx) == FALSE)
    
    data2.m <- beta.m[map.idx[rep.idx], , drop=FALSE]
    ref2.m <- ref.m[rep.idx, ]
    
    est.lm <- list()
    nui <- 1
    for (nu in nu.v) {
        est.m <- matrix(nrow = ncol(data2.m), ncol = ncol(ref2.m))
        colnames(est.m) <- colnames(ref2.m)
        rownames(est.m) <- colnames(data2.m)
        for (s in seq_len(ncol(data2.m))) {
            svm.o <- svm(x = ref2.m, y = data2.m[, s], scale = TRUE, type = "nu-regression", 
                kernel = "linear", nu = nu)
            coef.v <- t(svm.o$coefs) %*% svm.o$SV
            coef.v[which(coef.v < 0)] <- 0
            total <- sum(coef.v)
            coef.v <- coef.v/total
            est.m[s, ] <- coef.v
        }
        est.lm[[nui]] <- est.m
        nui <- nui + 1
    }
    
    #### select best nu
    rmse.m <- matrix(NA, nrow = ncol(beta.m), ncol = length(nu.v))
    for (nui in seq_along(nu.v)) {
        reconst.m <- ref2.m %*% t(est.lm[[nui]])
        s <- seq_len(ncol(beta.m))
        rmse.m[s, nui] <- sqrt(colMeans((data2.m[, s, drop = FALSE] - reconst.m[, s,  drop = FALSE])^2))
        message(nui)
    }
    colnames(rmse.m) <- nu.v
    nu.idx <- apply(rmse.m, 1, which.min)
    estF.m <- est.m
    for (s in seq_len(nrow(estF.m))) {
        estF.m[s, ] <- est.lm[[nu.idx[s]]][s, ]
    }
    return(list(estF = estF.m, nu = nu.v[nu.idx], ref = ref2.m, dataREF = data2.m))
}

#' @importFrom quadprog solve.QP   
### Houseman CP
DoCP <- function(beta.m, ref.m, constraint) {

    nCT <- ncol(ref.m)

    ### for inequality and equality, coe.v is different
    if (constraint == "inequality") {
        coe.v <- c(-1, 0)
    } else coe.v <- c(1, 1)
    
    ### define constraints
    A.m <- matrix(0, nrow = nCT, ncol = nCT)
    diag(A.m) <- rep(1, nCT)
    A.m <- cbind(rep(coe.v[1], nCT), A.m)
    b0.v <- c(coe.v[1], rep(0, nCT))
    
    ### define d-vector and solve for each sample
    nS <- ncol(beta.m)
    westQP.m <- matrix(NA, ncol = ncol(ref.m), nrow = nS)
    colnames(westQP.m) <- colnames(ref.m)
    rownames(westQP.m) <- colnames(beta.m)
    
    ### define D matrix
    map.idx <- match(rownames(ref.m), rownames(beta.m))
    rep.idx <- which(is.na(map.idx) == FALSE)
    D <- 2 * apply(ref.m[rep.idx,], 2, function(x) colSums(x * ref.m[rep.idx,]))
                   
    for (s in seq_len(nS)) {
        tmp.v <- beta.m[, s]
        d.v <- as.vector(2 * matrix(tmp.v[map.idx[rep.idx]], nrow = 1) %*% ref.m[rep.idx, 
            ])
        qp.o <- solve.QP(D, d.v, A.m, b0.v, meq = coe.v[2])
        westQP.m[s, ] <- qp.o$sol
        message(s)
    }
    
    return(list(estF = westQP.m, ref = ref.m[rep.idx, ], dataREF = beta.m[map.idx[rep.idx], 
        ]))
}

context("epidish")

test_that("# of columns in output should equal to that of reference", {
    
    data(centDHSbloodDMC.m)
    data(DummyBeta.m)
    idx <- sample(1:7, sample(2:7, 1), replace = FALSE)
    ref.m <- centDHSbloodDMC.m[, idx]
    estF.m <- epidish(DummyBeta.m, ref.m, method = "RPC")$estF
    
    dim1 <- ncol(ref.m)
    dim2 <- ncol(estF.m)
    expect_identical(dim1, dim2)
})

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/LiuDataSub.m.R
\docType{data}
\name{LiuDataSub.m}
\alias{LiuDataSub.m}
\title{Whole blood example beta value matrix}
\format{
A matrix with 500 CpGs and 50 columns
}
\usage{
data(LiuDataSub.m)
}
\description{
This beta value matrix is a subset matrix of Liu et al data(GSE42861). Beta 
values of 326 CpGs in the \code{centDHSbloodDMC} reference matrix and other 
randomly chosen 174 CpGs are included for 50 randomly chosen samples.
}
\details{
\itemize{
  \item beta value matrix of 500 CpGs and 50 samples
}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/epidish.R
\name{epidish}
\alias{epidish}
\title{Epigenetic Dissection of Intra-Sample-Heterogeneity}
\usage{
epidish(
  beta.m,
  ref.m,
  method = c("RPC", "CBS", "CP"),
  maxit = 50,
  nu.v = c(0.25, 0.5, 0.75),
  constraint = c("inequality", "equality")
)
}
\arguments{
\item{beta.m}{A data matrix with rows labeling the molecular features (should use same ID 
as in ref.m) and columns labeling samples (e.g. primary tumour specimens). 
Missing value is not allowed and all values should be positive or zero. 
In the case of DNA methylation, these are beta-values.}

\item{ref.m}{A matrix of reference 'centroids', i.e. representative molecular profiles, 
for a number of cell subtypes. rows label molecular features (e.g. CpGs,...) 
and columns label the cell-type. IDs need to be provided as rownames and 
colnames, respectively. Missing value is not allowed, and all values in 
this matrix should be positive or zero. For DNAm data, values should be 
beta-values.}

\item{method}{Chioce of a reference-based method ('RPC','CBS','CP')}

\item{maxit}{Only used in RPC mode, the limit of the number of IWLS iterations}

\item{nu.v}{Only used in CBS mode. It is a vector of several candidate nu values. nu is 
parameter needed for nu-classification, nu-regression, and 
one-classification in svm. The best estimation results among all candidate nu 
will be automatically returned.}

\item{constraint}{Only used in CP mode, you can choose either of 'inequality' or 'equality' 
normalization constraint. The default is 'inequality' (i.e sum of weights 
adds to a number less or equal than 1), which was implemented in 
Houseman et al (2012).}
}
\value{
CP-mode
A list with the following entries: estF: a matrix of the estimated fractions; 
ref: the reference centroid matrix used; dataREF: the subset of the input 
data matrix with only the probes defined in the reference matrix.

CBS-mode
A list with the following entries: estF: a matrix of the estimated fractions; 
nu: a vector of 'best' nu-parameter for each sample; 
ref: the reference centroid matrix used;
dataREF: the subset of the input data matrix with only the probes defined in the 
reference matrix.

RPC-mode
A list with the following entries: estF: a matrix of the estimated fractions;
 ref: the reference centroid matrix used; 
dataREF: the subset of the input data matrix with only the probes defined in the 
reference matrix.
}
\description{
A reference-based function to infer the fractions of a priori known cell 
subtypes present in a sample representing a mixture of such cell-types. 
Inference proceeds via one of 3 methods (Robust Partial Correlations-RPC, 
Cibersort-CBS, Constrained Projection-CP), as determined by the user.
}
\examples{
data(centDHSbloodDMC.m)
data(DummyBeta.m)
out.l <- epidish(DummyBeta.m, centDHSbloodDMC.m[,1:6], method = 'RPC')
frac.m <- out.l$estF


}
\references{
Teschendorff AE, Breeze CE, Zheng SC, Beck S. 
\emph{A comparison of reference-based algorithms for correcting cell-type 
heterogeneity in Epigenome-Wide Association Studies.}
BMC Bioinformatics (2017) 18: 105.
doi:\href{https://doi.org/10.1186/s12859-017-1511-5}{
10.1186/s12859-017-1511-5}.

Houseman EA, Accomando WP, Koestler DC, Christensen BC, Marsit CJ, 
Nelson HH, Wiencke JK, Kelsey KT. 
\emph{DNA methylation arrays as surrogate measures of cell mixture 
distribution.} 
BMC Bioinformatics (2012) 13: 86.
doi:\href{https://doi.org/10.1186/1471-2105-13-86}{10.1186/1471-2105-13-86}.

Newman AM, Liu CL, Green MR, Gentles AJ, Feng W, Xu Y, Hoang CD, Diehn M, 
Alizadeh AA. 
\emph{Robust enumeration of cell subsets from tissue expression profiles.}
Nat Methods (2015) 12: 453-457.
doi:\href{https://doi.org/10.1038/nmeth.3337}{10.1038/nmeth.3337}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/centBloodSub.m.R
\docType{data}
\name{centBloodSub.m}
\alias{centBloodSub.m}
\title{Whole blood reference of 188 tsDHS-DMCs and 7 blood cell subtypes}
\format{
A matrix with 188 rows and 7 columns
}
\usage{
data(centBloodSub.m)
}
\description{
This reference is a subset of \code{centDHSbloodDMC.m}, and contains 188 
DMCs which exhibit similar median DNAm values across epithelial cells, 
fibroblasts and ICs to ensure that the estimation of IC subtype fractions is 
not confounded by the epithelial and fibroblast cells in the sample. It should 
be used in the \code{hepidish} function to estimate fractions of immunce cell 
 subtypes.
}
\details{
\itemize{
  \item B-cells
  \item CD4+ T-cells
  \item CD8+ T-cells 
  \item NK-cells
  \item Monocytes
  \item Neutrophils 
  \item Eosinophils
}
}
\references{
Zheng SC, Webster AP, Dong D, Feber A, Graham DG, Sullivan R, Jevons S, Lovat LB, 
Beck S, Widschwendter M, Teschendorff AE
\emph{A novel cell-type deconvolution algorithm reveals substantial contamination by immune cells in saliva, buccal and cervix.}
Epigenomics (2018) 10: 925-940.
doi:\href{https://doi.org/10.2217/epi-2018-0037}{
10.2217/epi-2018-0037}.

Teschendorff AE, Breeze CE, Zheng SC, Beck S. 
\emph{A comparison of reference-based algorithms for correcting cell-type 
heterogeneity in Epigenome-Wide Association Studies.}
BMC Bioinformatics (2017) 18: 105.
doi:\href{https://doi.org/10.1186/s12859-017-1511-5}{
10.1186/s12859-017-1511-5}.

Reinius LE, Acevedo N, Joerink M, Pershagen G, Dahlen S-E, Greco D, 
Soderhall C, Scheynius A, Kere J.
\emph{Differential DNA methylation in purified human blood cells: 
implications for cell lineage and studies on disease susceptibility.}
PLoS ONE (2012) 7: e41361.
doi:\href{https://doi.org/10.1371/journal.pone.0041361}{
10.1371/journal.pone.0041361}.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/centEpiFibIC.m.R
\docType{data}
\name{centEpiFibIC.m}
\alias{centEpiFibIC.m}
\title{Reference for genenric epithelial tissue}
\format{
A matrix with 716 rows and 3 columns
}
\usage{
data(centEpiFibIC.m)
}
\description{
This reference could be used to estimate pproportions of epithelial cells, 
fibroblasts, and total immune cells in epithelial tissues.
}
\details{
\itemize{
  \item Epi
  \item Fib
  \item IC
}
}
\references{
Zheng SC, Webster AP, Dong D, Feber A, Graham DG, Sullivan R, Jevons S, Lovat LB, 
Beck S, Widschwendter M, Teschendorff AE
\emph{A novel cell-type deconvolution algorithm reveals substantial contamination by immune cells in saliva, buccal and cervix.}
Epigenomics (2018) 10: 925-940.
doi:\href{https://doi.org/10.2217/epi-2018-0037}{
10.2217/epi-2018-0037}.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CellDMC.R
\name{CellDMC}
\alias{CellDMC}
\title{A function that allows the identification of differentially methylated 
cell-types in in Epigenome-Wide Association Studies(EWAS)}
\usage{
CellDMC(
  beta.m,
  pheno.v,
  frac.m,
  adjPMethod = "fdr",
  adjPThresh = 0.05,
  cov.mod = NULL,
  sort = FALSE,
  mc.cores = 1
)
}
\arguments{
\item{beta.m}{A beta value matrix with rows labeling the CpGs and columns labeling 
samples.}

\item{pheno.v}{A vector of phenotype. CellDMC can handle both of binary and 
continuous/oderinal phenotypes. \code{NA} is not allowed in
\code{pheno.v}.}

\item{frac.m}{A matrix contains fractions of each cell-type. Each row labels a sample, 
with the same order of the columns in beta.m. Each column labels a 
cell-type. Column names, which are the names of cell-types, are required. The 
rowSums of frac.m should be 1 or close to 1.}

\item{adjPMethod}{The method used to adjust p values. The method can be any of method
accepted by \code{\link{p.adjust}}.}

\item{adjPThresh}{A numeric value, default as 0.05. This is used to call DMCTs. 
For each cell-type respectively, the CpG with the adjusted p values less than
 this threshold will be reported as DMCTs (-1 or 1) in the 'dmct' matrix in 
 the returned list.}

\item{cov.mod}{A design matrix from \code{model.matrix}, which contains other covariates to
be adjusted. For example, input 
\code{model.matrix(~ geneder, data = pheno.df)} to adjust gender. Do not put
 cell-type fraction here!}

\item{sort}{Default as \code{FALSE}. If \code{TRUE}, the data.frame in coe list will be
sorted based on p value of each CpG. The order of rows in 'dmct' will not 
change since the orders of each cell-type are different.}

\item{mc.cores}{The number of cores to use, i.e. at most how many threads will 
run simultaneously. The defatul is 1, which means no parallelization.}
}
\value{
A list with the following two items.

dmct
A matrix gives wheter the input CpGs are DMCTs and DMCs. The first column 
tells whether a CpG is a DMC or not. If the CpG is called as DMC, the value 
will be 1, otherwise it is 0. The following columns give DMCTs for each 
cell-type. If a CpG is a DMCT, the value will be 1 (hypermethylated for case 
compared to control) or -1 (hypomethylated for case compared to control). 
Otherwise, the value is 0 (non-DMCT). The rows of this matrix are ordered as
the same as that of the input \code{beta.m}.

coe
This list contains several dataframes, which correspond to each cell-type in
 \code{frac.m}. Each dataframe contains all CpGs in input \code{beta.m}. 
 All dataframes contain estimated DNAm changes (\code{Estimate}), 
 standard error (\code{SE}), estimated t statistics (\code{t}), 
 raw P values (\code{p}), and multiple hypothesis corrected P 
 values (\code{adjP}).
}
\description{
An outstanding challenge of Epigenome-Wide Association Studies performed in 
complex tissues is the identification of the specific cell-type(s) 
responsible for the observed differential methylation. CellDMC is a novel 
statistical algorithm, which is able to identify not only differentially 
methylated positions, but also the specific cell-type(s) driving the 
methylation change.
}
\examples{
data(centEpiFibIC.m)
data(DummyBeta.m)
out.l <- epidish(DummyBeta.m, centEpiFibIC.m, method = 'RPC')
frac.m <- out.l$estF
pheno.v <- rep(c(0, 1), each = 5)
celldmc.o <- CellDMC(DummyBeta.m, pheno.v, frac.m) 
# Pls note this is a faked beta value matrix.


}
\references{
Zheng SC, Breeze CE, Beck S, Teschendorff AE. 
\emph{Identification of differentially methylated cell-types in 
Epigenome-Wide Association Studies.} 
Nat Methods (2018) 15: 1059-1066
doi:\href{https://doi.org/10.1038/s41592-018-0213-x}{10.1038/s41592-018-0213-x}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ConstructDNAmPanel.R
\name{ConstructDNAmPanel}
\alias{ConstructDNAmPanel}
\title{Construct DNA Methylation reference Panel}
\usage{
ConstructDNAmPanel(
  dnam.matrix,
  cell.types.info,
  panel.cell.types,
  n.probes = 100,
  fdr.threshold = 0.05,
  p.adjust.method = "BH",
  summary.method = "mean",
  equal.variance = FALSE
)
}
\arguments{
\item{dnam.matrix}{A DNA methylation beta value matrix with no NA values.
Rows should be Probe IDs and columns should be samples.}

\item{cell.types.info}{A data frame that must contain a 'CellType' column,
specifying the cell type for each sample in `dnam.matrix`.}

\item{panel.cell.types}{A character vector specifying for which cell types a
panel should be constructed. These must be present in
`cell.types.info$CellType`.}

\item{n.probes}{The number of top-ranked hypo-methylated probes to select for
each cell type. Default is 100.}

\item{fdr.threshold}{The FDR (False Discovery Rate) threshold for filtering
significantly differentially methylated probes. Default is 0.05.}

\item{p.adjust.method}{The method for multiple testing correction. Must be one of
the methods supported by `stats::p.adjust`. Default is "BH".}

\item{summary.method}{The method to use for summarizing methylation values when
creating the reference panel. Can be "mean" or "median".
Default is "mean".}

\item{equal.variance}{A logical value (`TRUE` or `FALSE`) passed to `genefilter::fastT`, 
indicating whether to assume equal variance between groups. Default is FALSE.}
}
\value{
A list containing two components:
        \item{DetailedResults}{A list where each element is a data frame for a
                               cell type in `panel.cell.types`. The data frame
                               contains detailed information for the selected probes,
                               such as GapScore, AbsEffectSize, Zstatistic, and Pvalue.}
        \item{ReferencePanel}{A matrix with probes as rows and cell types as columns.
                              The values are the mean or median methylation beta values
                              for each probe in the corresponding cell type. This can serve
                              as a reference matrix for deconvolution.}
}
\description{
Constructs a cell-type-specific DNA methylation panel by selecting
             top-ranked hypo-methylated probes based on a "Gap Specificity Score" metric,
             which prioritizes markers with large effect sizes.
}
\details{
The function's logic is as follows:
1. For each target cell type, it is compared against all other cell types.
2. A t-test is used to find differentially methylated probes, and p-values are
   adjusted to FDR using the specified `p.adjust.method`.
3. Probes are filtered based on the `fdr.threshold`.
4. Only probes that are hypo-methylated in the target cell type (negative t-statistic)
   are retained.
5. A "Gap specificity Score" is calculated for these probes (defined as the minimum beta value
   in other cell types minus the maximum beta value in the target cell type).
6. Probes are ranked by the Gap Score, and the top `n.probes` are selected.
7. Finally, the probes selected for all target cell types are combined to form the
   final panel, and the reference matrix is computed using the specified `summary.method`.
}
\examples{
\dontrun{
# Assuming you have referenceBetas_matrix_noNA and referenceCovars_df
# library(genefilter)
# library(stats)
# library(matrixStats)

# Define target cell types for the panel
target_cell_types <- c("CD4T", "Bcell", "Mono")

# Construct the Panel
panel_results <- ConstructDNAmPanel(
  dnam.matrix = referenceBetas_matrix_noNA,
  cell.types.info = referenceCovars_df,
  panel.cell.types = target_cell_types,
  n.probes = 50,
  fdr.threshold = 0.05,
  p.adjust.method = "BH",
  summary.method = "median" 
)

# View the results
# View the reference panel matrix
head(panel_results$ReferencePanel)

# View detailed probe info for B-cells
head(panel_results$DetailedResults$Bcell)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/centEpiFibFatIC.m.R
\docType{data}
\name{centEpiFibFatIC.m}
\alias{centEpiFibFatIC.m}
\title{Reference for breast tissue}
\format{
A matrix with 491 rows and 4 columns
}
\usage{
data(centEpiFibFatIC.m)
}
\description{
This reference was designed for estimating fractions of epithelial 
cells, fibroblasts, fat cells and total immune cells in breast tissue.
}
\details{
\itemize{
  \item Epi
  \item Fib
  \item Fat
  \item IC
}
}
\references{
Zheng SC, Webster AP, Dong D, Feber A, Graham DG, Sullivan R, Jevons S, Lovat LB, 
Beck S, Widschwendter M, Teschendorff AE
\emph{A novel cell-type deconvolution algorithm reveals substantial contamination by immune cells in saliva, buccal and cervix.}
Epigenomics (2018) 10: 925-940.
doi:\href{https://doi.org/10.2217/epi-2018-0037}{
10.2217/epi-2018-0037}.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hepidish.R
\name{hepidish}
\alias{hepidish}
\title{Hierarchical EpiDISH (HEpiDISH)}
\usage{
hepidish(
  beta.m,
  ref1.m,
  ref2.m,
  h.CT.idx,
  method = c("RPC", "CBS", "CP"),
  maxit = 50,
  nu.v = c(0.25, 0.5, 0.75),
  constraint = c("inequality", "equality")
)
}
\arguments{
\item{beta.m}{A data matrix with rows labeling the molecular features (should use same ID 
as in reference matrices) and columns labeling samples (e.g. primary tumour specimens). 
Missing value is not allowed and all values should be positive or zero. 
In the case of DNA methylation, these are beta-values.}

\item{ref1.m}{A matrix of \strong{primary} reference 'centroids', i.e. representative molecular profiles, 
for a number of cell subtypes. rows label molecular features (e.g. CpGs,...) 
and columns label the cell-type. IDs need to be provided as rownames and 
colnames, respectively. Missing value is not allowed, and all values in 
this matrix should be positive or zero. For DNAm data, values should be 
beta-values.}

\item{ref2.m}{Similar to \code{ref1.m}, but now a A matrix of \strong{secondary} reference.
For example, \code{ref1.m} contains reference centroids for epithelial cells,
fibroblasts and total immune cells. \code{ref2.m} can be subtypes of immune
cells, such as B-cells, NK cells, monocytes and etc.}

\item{h.CT.idx}{A index tells which cell-type in \code{ref1.m} is the higher order cell-types 
in \code{ref2.m}. For example, \code{ref1.m} contains reference centroids for 
epithelial cells, fibroblasts and total immune cells. \code{ref2.m} contains 
subtypes of immune cells, the \code{h.CT.idx} should be 3, corresponding to
immune cells in \code{ref1.m}.}

\item{method}{Chioce of a reference-based method ('RPC','CBS','CP')}

\item{maxit}{Only used in RPC mode, the limit of the number of IWLS iterations}

\item{nu.v}{Only used in CBS mode. It is a vector of several candidate nu values. nu is 
parameter needed for nu-classification, nu-regression, and 
one-classification in svm. The best estimation results among all candidate nu 
will be automatically returned.}

\item{constraint}{Only used in CP mode, you can choose either of 'inequality' or 'equality' 
normalization constraint. The default is 'inequality' (i.e sum of weights 
adds to a number less or equal than 1), which was implemented in 
Houseman et al (2012).}
}
\value{
A matrix of the estimated fractions
}
\description{
HEpiDISH is an iterative hierarchical procedure of EpiDISH. HEpiDISH uses two 
distinct DNAm references, a primary reference for the estimation of several 
cell-types fractions, and a separate secondary non-overlapping DNAm reference 
for the estimation of underlying subtype fractions of one of the cell-type in 
the primary reference.
}
\examples{
data(centEpiFibIC.m)
data(centBloodSub.m)
data(DummyBeta.m)
frac.m <- hepidish(beta.m = DummyBeta.m, ref1.m = centEpiFibIC.m, 
ref2.m = centBloodSub.m, h.CT.idx = 3, method = 'RPC')


}
\references{
Zheng SC, Webster AP, Dong D, Feber A, Graham DG, Sullivan R, Jevons S, Lovat LB, 
Beck S, Widschwendter M, Teschendorff AE
\emph{A novel cell-type deconvolution algorithm reveals substantial contamination by immune cells in saliva, buccal and cervix.}
Epigenomics (2018) 10: 925-940.
doi:\href{https://doi.org/10.2217/epi-2018-0037}{
10.2217/epi-2018-0037}.

Teschendorff AE, Breeze CE, Zheng SC, Beck S. 
\emph{A comparison of reference-based algorithms for correcting cell-type 
heterogeneity in Epigenome-Wide Association Studies.}
BMC Bioinformatics (2017) 18: 105.
doi:\href{https://doi.org/10.1186/s12859-017-1511-5}{
10.1186/s12859-017-1511-5}.

Houseman EA, Accomando WP, Koestler DC, Christensen BC, Marsit CJ, 
Nelson HH, Wiencke JK, Kelsey KT. 
\emph{DNA methylation arrays as surrogate measures of cell mixture 
distribution.} 
BMC Bioinformatics (2012) 13: 86.
doi:\href{https://doi.org/10.1186/1471-2105-13-86}{10.1186/1471-2105-13-86}.

Newman AM, Liu CL, Green MR, Gentles AJ, Feng W, Xu Y, Hoang CD, Diehn M, 
Alizadeh AA. 
\emph{Robust enumeration of cell subsets from tissue expression profiles.}
Nat Methods (2015) 12: 453-457.
doi:\href{https://doi.org/10.1038/nmeth.3337}{10.1038/nmeth.3337}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cent12CT450k.R
\docType{data}
\name{cent12CT450k.m}
\alias{cent12CT450k.m}
\title{Whole blood reference of 12 blood cell subtypes for 450k array}
\format{
A matrix with 600 rows and 12 columns
}
\usage{
data(cent12CT450k.m)
}
\description{
This reference is constructed using data from Salas et al.(2022). It contains
 the following 12 blood cell subtypes. This reference is constructed for 450k arrays.
 See \code{cent12CT.m} for EPIC arrays.
}
\details{
\itemize{
  \item CD4+ naive T-cells
  \item Basophil cells
  \item CD4+ memory T-cells
  \item Memory B-cells
  \item Naive B-cells
  \item Regulatory T-Cells
  \item CD8+ memory T-cells
  \item CD8+ naive T-cells
  \item Eosinophils
  \item NK-cells
  \item Neutrophils 
  \item Monocytes
}
}
\references{
Qi Luo, Varun B Dwaraka, Qingwen Chen, Huige Tong, Tianyu Zhu,Kirsten
Seale, Joseph M Raffaele, Shijie C Zheng, Tavis L Mendez, Yulu Chen,
Natalia Carreras, Sofina Begum, Kevin Mendez, Sarah Voisin, Nir Eynon,
Jessica A Lasky-Su, Ryan Smith, Andrew E Teschendorff. \emph{A
  meta-analysis of immune-cell fractions at high resolution reveals
  novel associations with common phenotypes and health outcomes.} Genome
Med. (2023) 15(1): 59.
doi:\href{https://doi.org/10.1186/s13073-023-01211-5}{10.1186/s13073-023-01211-5}.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cent12CT.R
\docType{data}
\name{centUniLIFE.m}
\alias{centUniLIFE.m}
\title{DNAm reference matrix for 19 immune cell-types for blood of any age}
\format{
A matrix with 1906 CpGs (rows) and 19 immune cell-types (columns)
}
\usage{
data(centUniLIFE.m)
}
\description{
This reference is constructed using data from Salas et al.(2022) and
several cord-blood DNAm datasets. It contains the following 19 immune cell
subtypes. This reference can be used for EPIC and 450k arrays.
}
\details{
\itemize{
  \item cord-blood granulocytes
  \item cord-blood monocytes
  \item cord-blood nRBCs
  \item cord-blood B-cells
  \item cord-blood NK-cells
  \item cord-blood CD8+ T-cells
  \item cord-blood CD4+ T-cells
  \item adult CD4+ naive T-cells
  \item adult Basophil cells
  \item adult CD4+ memory T-cells
  \item adult Memory B-cells
  \item adult Naive B-cells
  \item adult Regulatory T-Cells
  \item adult CD8+ memory T-cells
  \item adult CD8+ naive T-cells
  \item adult Eosinophils
  \item adult NK-cells
  \item adult Neutrophils 
  \item adult Monocytes
}
}
\references{
Xiaolong Guo et al \emph{Universal high resolution cell-type
  deconvolution of blood tissue from any age}. Submitted.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DoMetaEfron.R
\name{DoMetaEfron}
\alias{DoMetaEfron}
\title{An R-function to perform a meta-analysis over multiple studies using 
an empirical Bayes procedure by Efron followed by Stouffer method.}
\usage{
DoMetaEfron(stat.m, pval.m, bre = 120, df = 15, pct0 = 0.25, plotlocfdr = 0)
}
\arguments{
\item{stat.m}{A matrix of signed statistics (e.g. t-statistics) with rows labeling genomic 
features (e.g. CpGs or genes) and columns labeling studies. rownames must be 
provided.}

\item{pval.m}{A matrix matched to stat.m containing the associated P-values, with rows 
labeling genomic features (e.g. CpGs or genes) and columns labeling studies.}

\item{bre}{The number of breakpoints to divide statistics per study into bins. By 
default this is 120. See input argument for locfdr function from locfdr 
package.}

\item{df}{The number of degrees of freedom for fitting spline. By default this is 
15. See input argument for locfdr function from locfdr package.}

\item{pct0}{Percentage of statistics to use for fitting null. By default this is 0.25 
(i.e. 25\%).}

\item{plotlocfdr}{Determines whether to plot output or not. By default this is set to 0 meaning
no plot. See input argument for locfdr function from locfdr package.}
}
\value{
meta.m
A matrix with rows as in stat.m, and with 3 columns labeling Stouffer's 
z-statistic, P-value and Benjamini-Hochberg adjusted P-value.
}
\description{
An R-function to perform a meta-analysis over multiple studies using 
an empirical Bayes procedure by Efron followed by Stouffer method.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DummyBeta.m.R
\docType{data}
\name{DummyBeta.m}
\alias{DummyBeta.m}
\title{Dummy beta value matrix}
\format{
A matrix with 2000 CpGs and 10 columns
}
\usage{
data(DummyBeta.m)
}
\description{
A faked beta value matrix of 2000 CpGs and 10 samples. 
Only used for demostration purpose..
}
\details{
\itemize{
  \item beta value matrix of 2000 CpGs and 10 samples
}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/centDHSbloodDMC.m.R
\docType{data}
\name{centDHSbloodDMC.m}
\alias{centDHSbloodDMC.m}
\title{Whole blood reference of 333 tsDHS-DMCs and 7 blood cell subtypes}
\format{
A matrix with 333 rows and 7 columns
}
\usage{
data(centDHSbloodDMC.m)
}
\description{
Reference-based cell-type fraction estimation algorithms rely on a prior 
defined reference matrix. We leveraged cell-type specific DNAse Hypersensitive 
Site (DHS) information from the NIH Epigenomics Roadmap, and used 450k purified 
blood cell types dataset from Reinius et al (2012) to  construct this 
improved whole blood reference DNA methylation dataset, as described in 
Teschendorff et al (2017). It contains 333 tsDHS-DMCs of 7 blood cell 
subtypes(\emph{As the fractions of eosinophils are usually small, you could 
add the estimated fractions of neutrophils and eosinophils togetther as the 
estimations of granulocytes.}):
}
\details{
\itemize{
  \item B-cells
  \item CD4+ T-cells
  \item CD8+ T-cells 
  \item NK-cells
  \item Monocytes
  \item Neutrophils 
  \item Eosinophils
}
}
\references{
Teschendorff AE, Breeze CE, Zheng SC, Beck S. 
\emph{A comparison of reference-based algorithms for correcting cell-type 
heterogeneity in Epigenome-Wide Association Studies.}
BMC Bioinformatics (2017) 18: 105.
doi:\href{https://doi.org/10.1186/s12859-017-1511-5}{
10.1186/s12859-017-1511-5}.

Reinius LE, Acevedo N, Joerink M, Pershagen G, Dahlen S-E, Greco D, 
Soderhall C, Scheynius A, Kere J.
\emph{Differential DNA methylation in purified human blood cells: 
implications for cell lineage and studies on disease susceptibility.}
PLoS ONE (2012) 7: e41361.
doi:\href{https://doi.org/10.1371/journal.pone.0041361}{
10.1371/journal.pone.0041361}.
}
\keyword{datasets}
